@@ -1,7 +1,7 @@
 # This file is part of Supysonic.
 # Supysonic is a Python implementation of the Subsonic server API.
 #
-# Copyright (C) 2013-2022 Alban 'spl0k' Féron
+# Copyright (C) 2013-2023 Alban 'spl0k' Féron
 #
 # Distributed under terms of the GNU AGPLv3 license.
 
@@ -15,7 +15,6 @@ from datetime import datetime
 from hashlib import sha1
 from peewee import (
     AutoField,
-    BinaryUUIDField,
     BlobField,
     BooleanField,
     CharField,
@@ -24,6 +23,7 @@ from peewee import (
     ForeignKeyField,
     IntegerField,
     TextField,
+    UUIDField,
 )
 from peewee import CompositeKey, DatabaseProxy, Model, MySQLDatabase
 from peewee import fn
@@ -31,7 +31,7 @@ from playhouse.db_url import parseresult_to_dict, schemes
 from urllib.parse import urlparse
 from uuid import UUID, uuid4
 
-SCHEMA_VERSION = "20200607"
+SCHEMA_VERSION = "20230115"
 
 
 def now():
@@ -45,7 +45,7 @@ def random():
 
 
 def PrimaryKeyField(**kwargs):
-    return BinaryUUIDField(primary_key=True, default=uuid4, **kwargs)
+    return UUIDField(primary_key=True, default=uuid4, **kwargs)
 
 
 db = DatabaseProxy()
@@ -167,18 +167,52 @@ class Folder(PathMixin, _Model):
 
     @classmethod
     def prune(cls):
-        query = cls.delete().where(
+        alias = cls.alias()
+        query = cls.select(cls.id).where(
             ~cls.root,
-            cls.id.not_in(Track.select(Track.folder)),
-            cls.id.not_in(cls.select(cls.parent)),
+            Track.select(fn.count("*")).where(Track.folder == cls.id) == 0,
+            alias.select(fn.count("*")).where(alias.parent == cls.id) == 0,
         )
         total = 0
         while True:
-            count = query.execute()
-            total += count
-            if not count:
+            clone = query.clone()  # peewee caches the results, clone to force a refetch
+            for f in clone:
+                f.delete_instance(recursive=True)
+                total += 1
+            if not len(clone):
                 return total
 
+    def delete_hierarchy(self):
+        if self.root:
+            cond = Track.root_folder == self
+        else:
+            cond = Track.path.startswith(self.path)
+
+        return self.__delete_hierarchy(cond)
+
+    def __delete_hierarchy(self, cond):
+        users = User.select(User.id).join(Track).where(cond)
+        User.update(last_play=None).where(User.id.in_(users)).execute()
+
+        tracks = Track.select(Track.id).where(cond)
+        RatingTrack.delete().where(RatingTrack.rated.in_(tracks)).execute()
+        StarredTrack.delete().where(StarredTrack.starred.in_(tracks)).execute()
+
+        path_cond = Folder.path.startswith(self.path)
+        folders = Folder.select(Folder.id).where(path_cond)
+        RatingFolder.delete().where(RatingFolder.rated.in_(folders)).execute()
+        StarredFolder.delete().where(StarredFolder.starred.in_(folders)).execute()
+
+        deleted_tracks = Track.delete().where(cond).execute()
+
+        query = Folder.delete().where(path_cond)
+        if isinstance(db.obj, MySQLDatabase):
+            # MySQL can't propery resolve deletion order when it has several to handle
+            query = query.order_by(Folder.path.desc())
+        query.execute()
+
+        return deleted_tracks
+
 
 class Artist(_Model):
     id = PrimaryKeyField()
@@ -202,11 +236,19 @@ class Artist(_Model):
 
     @classmethod
     def prune(cls):
+        album_artists = Album.select(Album.artist)
+        track_artists = Track.select(Track.artist)
+
+        StarredArtist.delete().where(
+            StarredArtist.starred.not_in(album_artists),
+            StarredArtist.starred.not_in(track_artists),
+        ).execute()
+
         return (
             cls.delete()
             .where(
-                cls.id.not_in(Album.select(Album.artist)),
-                cls.id.not_in(Track.select(Track.artist)),
+                cls.id.not_in(album_artists),
+                cls.id.not_in(track_artists),
             )
             .execute()
         )
@@ -269,7 +311,9 @@ class Album(_Model):
 
     @classmethod
     def prune(cls):
-        return cls.delete().where(cls.id.not_in(Track.select(Track.album))).execute()
+        albums = Track.select(Track.album)
+        StarredAlbum.delete().where(StarredAlbum.starred.not_in(albums)).execute()
+        return cls.delete().where(cls.id.not_in(albums)).execute()
 
 
 class Track(PathMixin, _Model):
@@ -588,6 +632,7 @@ def init_database(database_uri):
         provider = "postgres"
     elif uri.scheme.startswith("sqlite"):
         provider = "sqlite"
+        args["pragmas"] = {"foreign_keys": 1}
     else:
         raise RuntimeError(f"Unsupported database: {uri.scheme}")
 
@@ -597,8 +642,9 @@ def init_database(database_uri):
 
     # Check if we should create the tables
     if not db.table_exists("meta"):
-        execute_sql_resource_script(f"schema/{provider}.sql")
-        Meta.create(key="schema_version", value=SCHEMA_VERSION)
+        with db.atomic():
+            execute_sql_resource_script(f"schema/{provider}.sql")
+            Meta.create(key="schema_version", value=SCHEMA_VERSION)
 
     # Check for schema changes
     version = Meta["schema_version"]
@@ -607,11 +653,18 @@ def init_database(database_uri):
             pkg_resources.resource_listdir(__package__, f"schema/migration/{provider}")
         )
         for migration in migrations:
+            if migration[0] in ("_", "."):
+                continue
+
             date, ext = os.path.splitext(migration)
             if date <= version.value:
                 continue
+
             if ext == ".sql":
-                execute_sql_resource_script(f"schema/migration/{provider}/{migration}")
+                with db.atomic():
+                    execute_sql_resource_script(
+                        f"schema/migration/{provider}/{migration}"
+                    )
             elif ext == ".py":
                 m = importlib.import_module(
                     f".schema.migration.{provider}.{date}", __package__
